# ============================================================================
# Accessibility.py 
# count Accessibility index, (length of common borders)
# author: Katerina PAVKOVA, 2008
# ============================================================================

# import systems modules 
import sys, os, locale, arcgisscripting

# import helper functions
from HelperFunctions import *

# create geoprocessor object
gp = arcgisscripting.create()

# input
try:
    InPolygFC = sys.argv[1]                  # Feature Layer
except: raise ReportError (msgParseErr)

#-----------------------------------------------------------------------------------
# no records error
count = gp.GetCount(InPolygFC)                        # Counting records                                        
if count < 1:
        raise ReportError (msgNoRecsErr)

# feature type error 
description = gp.describe(InPolygFC)                  # describing shape type
if description.ShapeType <> "Polygon":                # when shype type is not polygon:
    raise ReportError(msgPolyOnlyErr)   

#-----------------------------------------------------------------------------------
# workspace
gp.workspace = "C:/temp/"
if not os.path.exists("C:/temp/"):
    if os.path.dirname(InPolygFC) <> "": gp.workspace = str(os.path.dirname(InPolygFC))
    else:
        message = ('\n''"C:/temp/" does not exist.'
                   '\nDo not load data as Feature Layer.\nPlease specify a path of Input Polygon Feature Class. \n ...'
                   '\n (Or rewrite workspace gp.workspace in the script.)\n\n')
        error = gp.GetMessages(2)
        OutMessage = message + "... " + error
        raise ReportError (OutMessage)
                    
#----------------------------------------------------------------------------------
# listing fields and getting their names
listFields = gp.ListFields(InPolygFC)
dscFields = []
Field = listFields.Next()                           # looping through list of Fields
while Field:                                        # to get their names
    name = Field.Name
    dscFields.append(name)
    Field = listFields.Next()

# checking if field exists in list of fields names, if not it is added 
indexField = "ai"
try:
    if indexField not in dscFields: gp.AddField_management(InPolygFC, indexField, "DOUBLE")
    else: gp.AddWarning("Values in existing field 'ai' will be rewrite")
except: raise gp.GetMessages(2)

#----------------------------------------------------------------------------------
# polygons to lines
lines = "linie.shp"
try:
    gp.PolygonToLine_management (InPolygFC, lines)
except: raise gp.GetMessage(2)

#====================================================================================
# counting index
# For shapefile expression, fields are double quoted (") and text values are single quoted (')
# for counting index we need just possitive numbers in LEFT_FID

selected = "selected.shp"
try:
    gp.select_analysis(lines, selected,' "LEFT_FID" >= 0 ')
    
except: raise gp.GetMessage(2)


# looping through polygons table
rowsP = gp.UpdateCursor(InPolygFC)
number = 0
rowP = rowsP.Next()
while rowP <> None:

    # looping through line table    
    rowsL = gp.UpdateCursor(selected)
    totalDist = 0
    rowL = rowsL.Next()
    while rowL <> None:
        if rowL.LEFT_FID == number:
            geometry = rowL.GetValue("Shape")
            dist = geometry.Length
            totalDist = totalDist + dist
        elif rowL.RIGHT_FID == number:
            geometry = rowL.GetValue("Shape")
            dist = geometry.Length
            totalDist = totalDist + dist
        rowL = rowsL.Next()
    rowsL = None

    rowP.SetValue(indexField, totalDist) 
    rowsP.UpdateRow(rowP)
    

    number = number + 1

    # displaying percentage of looping
    if number == round(count/4):
        gp.AddMessage('\n25% complete')
    if number == round(count/2):
        gp.AddMessage('50% complete')
    if number == round((count/4)*3):
        gp.AddMessage('75% complete ...')
    
    
    rowP = rowsP.Next()
rowsP = None

#----------------------------------------------------------------------------------------------
# deleting temporary files
try:
    gp.delete_management(selected)
    gp.delete_management(lines)
except: raise gp.GetMessage(2)

gp.AddMessage("\n Index values are in attribute table"'\n')

gp = None