# ============================================================================
# Diffusiveness.py 
# Vypocte index rozptylenpsti vsech plosek
# autor: Katerina PAVKOVA, 2008
# ============================================================================

# import systems modules
import arcgisscripting, string, math, sys, locale

# import helper functions
from HelperFunctions import *

# create geoprocessor object
gp = arcgisscripting.create()

# input
try:
    InPolygFC = sys.argv[1]                 # Polygon Feature Class
    InDistance = sys.argv[2]                # Double (optional), do jake vzdalenosti se maji brat sousedi
    if InDistance == "#" or InDistance == "": Distance = 1000
    else: Distance = locale.atof(InDistance)
except: raise ReportError (msgParseErr)

#-----------------------------------------------------------------------------------
# no records error
count = gp.GetCount(InPolygFC)                        # Counting records                                        
if count < 1:
        raise ReportError (msgNoRecsErr)

# feature type error 
description = gp.describe(InPolygFC)                  # describing shape type
if description.ShapeType <> "Polygon":              # when shype type is not polygon:
    raise ReportError(msgPolyOnlyErr)   

#---------------------------------------------------------------------------------
# workspace
gp.workspace = "C:/temp/"
if not os.path.exists("C:/temp/"):
    if os.path.dirname(InPolygFC) <> "": gp.workspace = str(os.path.dirname(InPolygFC))
    else:
        message = ('\n''"C:/temp/" does not exist.'
                   '\nDo not load data as Feature Layer.\nPlease specify a path of Input Polygon Feature Class. \n ...'
                   '\n (Or rewrite workspace gp.workspace in the script.)\n\n')
        error = gp.GetMessages(2)
        OutMessage = message + "... " + error
        raise ReportError (OutMessage)

#-----------------------------------------------------------------------------------
# polygons to centroids
centroids = "centroids.shp"
try:
    gp.FeatureToPoint_management(InPolygFC, centroids)   
except:
    raise gp.GetMessages(2)

#-------------------------------------------------------------------------------------
# funkce Near mezi stredy
try:
    gp.near_analysis(centroids, centroids, Distance, "LOCATION")
except:
    raise gp.GetMessages(2)

#=====================================================================================
# looping through centroids table

rows = gp.UpdateCursor(centroids)
row = rows.Next()
totalDist = 0
while row <> None:
    dist = row.NEAR_DIST
    totalDist = totalDist + dist
    row = rows.Next()
row = None

gp.AddMessage("\n40% complete")
#---------------------------------------------------------------------------------------
# looping through polygons table

rowsP = gp.UpdateCursor(InPolygFC)
rowP = rowsP.Next()
totalArea = 0
while rowP <> None:
    geometry = rowP.GetValue("Shape")
    area = geometry.Area
    totalArea = totalArea + area

    rowP = rowsP.Next()
rowsP = None

gp.AddMessage("80% complete ...")

#--------------------------------------------------------------------------------------
# counting index
# density is taken as quotient totalArea/count of polygons

meanDist = totalDist/count
density = totalArea/count

diffusiveness = 2*meanDist*(density/math.pi)

gp.AddMessage("\n ROZPTYLENOST PLOSEK"'\n'
              'v "' + str(InPolygFC) +'.shp":''\n'
              "Rc = " + str(diffusiveness) + '\n')

#--------------------------------------------------------------------------------------
# deleting temporary shape centroids.shp
try:
    gp.delete_management(centroids)
except:
    raise gp.GetMessages(2)
  
print "konec"