#===============================================================================
# Geometry.py
# calculate absolute and relative area , perimeter/length
# Author: Katerina PAVKOVA, 2008
#==============================================================================

# import systems module
import arcgisscripting, sys

# import helper functions
from HelperFunctions import *

# create geoprocessor object
gp = arcgisscripting.create()

#---------------------------------------------------------------------------
# input
try:
    inputFC = sys.argv[1]                           # Feature Class/Layer
except: raise ReportError (msgParseErr)

#--------------------------------------------------------------------------
# no records error
count = gp.GetCount(inputFC)                        # Counting records                                        
if count < 1:
        raise ReportError (msgNoRecsErr)

#-------------------------------------------------------------------
# listing fields and getting their names
listFields = gp.ListFields(inputFC)
infoFields = []
Field = listFields.Next()                           # looping through list of Fields
while Field:                                        # to get their names
    name = Field.Name
    infoFields.append(name)
    Field = listFields.Next()

#---------------------------------------------------------------------------
# getting shape info: 
description = gp.describe(inputFC)                  # describing shape type
if description.ShapeType == "Polygon":              # when shype type is polygon:           
    gp.AddMessage("\n Polygone Feature class \n")

    # adding fields and checking if field exists in list of fields names
    areaField = "Area_"
    if areaField not in infoFields: gp.AddField_management(inputFC, areaField, "DOUBLE")
    else: gp.AddWarning("Values in existing field 'Area_' will be rewrite")
    RelAreaField = "RelatArea"
    if RelAreaField not in infoFields: gp.AddField_management(inputFC, RelAreaField, "DOUBLE", 6, 3)
    else: gp.AddWarning("Values in existing field 'RelatArea_' will be rewrite")
    lengthField = "Perimeter_"
    if lengthField not in infoFields: gp.AddField_management(inputFC, lengthField, "DOUBLE")
    else: gp.AddWarning("Values in existing field 'Perimeter_' will be rewrite")

    # summarizing Area
    SumArea = 0
    rows1 = gp.UpdateCursor(inputFC)
    row1 = rows1.Next()
    while row1 <> None:
        geometry1 = row1.GetValue ("Shape")         # getting geometry info
        Area1 = geometry1.Area                      # getting Area value
        SumArea = SumArea + Area1
        row1 = rows1.Next()
    rows1 = None

    # calculating Relative Area, Area and Perimeter            
    rows = gp.UpdateCursor(inputFC)
    row = rows.Next()
    while row <> None:
        geometry = row.GetValue ("Shape")
        Area = geometry.Area
        RelArea = Area/SumArea*100        
        Perimeter = geometry.Length
        row.SetValue(areaField, Area)               # setting value into table
        row.SetValue(RelAreaField, RelArea)
        row.SetValue(lengthField, Perimeter)
        rows.UpdateRow(row)
        row = rows.Next()
    rows = None

elif description.ShapeType == "Polyline":           # when shype type is polyline:
    gp.AddMessage("\n Polyline Feature Class  \n") 

    # adding field and checking if field exists in list of fields names
    lengthField2 = "Length_"
    if lengthField2 not in infoFields: gp.AddField_management(inputFC, lengthField2, "DOUBLE")
    else: gp.AddWarning("\n Values in existing field 'Length_' will be rewrite \n")

    #loopint through rows and getting geometry info     
    rows2 = gp.UpdateCursor(inputFC)
    row2 = rows2.Next()
    while row2 <> None:
        geometry = row2.GetValue ("Shape")
        Length = geometry.Length
        row2.SetValue(lengthField2, Length)
        rows2.UpdateRow(row2)
        row2 = rows2.Next()
    rows2 = None

else:     # everything else is an error:
    raise ReportError("Input Feature Class must be Polygone or Polyline \n")         # FC must be polygon or polyline
    raise gp.AddMessage(2)

# final message for users
gp.AddMessage("\n Values are in attribute table \n")

gp = None