#===========================================================================
# Interaction.py 
# count index interaction between spots (polygons), neighbour distance is taken by their centroids, not between borders
# author: Katerina PAVKOVA, 2008
#===========================================================================

# import systems modules 
import sys, os, locale, arcgisscripting

# import helper functions
from HelperFunctions import *

# create geoprocessor object
gp = arcgisscripting.create()

# input
try:
    InPolygFC = sys.argv[1]                  # Feature Layer
    InDistance = sys.argv[2]                 # Double (optional), to what distance are another polygons taken like neighbours
    if InDistance == "#" or InDistance == "": Distance = 1000
    else: Distance = locale.atof(InDistance)
except: raise ReportError (msgParseErr)

#-----------------------------------------------------------------------------------
# no records error
count = gp.GetCount(InPolygFC)                        # Counting records                                        
if count < 1:
        raise ReportError (msgNoRecsErr)

# feature type error 
description = gp.describe(InPolygFC)                  # describing shape type
if description.ShapeType <> "Polygon":              # when shype type is not polygon:
    raise ReportError(msgPolyOnlyErr)   

#---------------------------------------------------------------------------------
# workspace
gp.workspace = "C:/temp/"
if not os.path.exists("C:/temp/"):
    if os.path.dirname(InPolygFC) <> "": gp.workspace = str(os.path.dirname(InPolygFC))
    else:
        message = ('\n''"C:/temp/" does not exist.'
                   '\nDo not load data as Feature Layer.\nPlease specify a path of Input Polygon Feature Class. \n ...'
                   '\n (Or rewrite workspace gp.workspace in the script.)\n\n')
        error = gp.GetMessages(2)
        OutMessage = message + "... " + error
        raise ReportError (OutMessage)

#----------------------------------------------------------------------------------
# listing fields and getting their names
listFields = gp.ListFields(InPolygFC)
dscFields = []
Field = listFields.Next()                           # looping through list of Fields
while Field:                                        # to get their names
    name = Field.Name
    dscFields.append(name)
    Field = listFields.Next()

# checking if field exists in list of fields names, if not it is added 
indexField = "Ii"
try:
    if indexField not in dscFields: gp.AddField_management(InPolygFC, indexField, "DOUBLE")
    else: gp.AddWarning("Values in existing field 'Ii' will be rewrite")
except: raise gp.GetMessages(2)

#--------------------------------------------------------------------
# polygons to centroids
centroids = "centroids.shp"
try:
    gp.FeatureToPoint_management(InPolygFC, centroids)   
except:
    raise gp.GetMessages(2)

#---------------------------------------------------------------------
# generating distance table between neighbours
# distance table has its specific view
Table = "neighbours.dbf"
gp.PointDistance_Analysis(centroids, centroids, Table, Distance)


#========================================================================
# counting index
sum = 0

# looping through table to get first value, 
rows = gp.UpdateCursor(Table)
row = rows.Next()
while row:
    start1 = row.INPUT_FID                     # getting first value
    row = rows.Reset()                         # returning back to first row
rows = None


# again from beginning looping through table
# comparsig if next values are equal to the first
rows1 = gp.UpdateCursor(Table)
number = 1
row1 = rows1.Next()
while row1:
    if start1 == row1.INPUT_FID:                 # if first value is equal to next value in table:
        neighbour = row1.NEAR_FID
        dist = row1.DISTANCE

        # looping through input polygon FC table, to get Shape geometry and area value            
        rowsP = gp.UpdateCursor(InPolygFC)
        rowP = rowsP.Next()
        while rowP:
            if neighbour == rowP.FID:
                geometry = rowP.GetValue ("Shape")
                area = geometry.Area
            rowP = rowsP.Next()
        rowsP = None

        # if distance is not zero: count index fraction and final sum
        if not dist == 0:       
            fraction = area/dist**2.0
            sum = sum + fraction

    else:                                       
        if sum <> 0:
            
            # recording index value to polygone attribute table        
            rows2 = gp.UpdateCursor(InPolygFC) 
            row2 = rows2.Next()
            while row2:
                if row2.FID == start1:        # to the same line like value start1
                    row2.SetValue(indexField, sum)

                    # displaying percentage of looping
                    if number == round(count/4):
                        gp.AddMessage('\n25% complete')
                    if number == round(count/2):
                        gp.AddMessage('50% complete')
                    if number == round((count/4)*3):
                        gp.AddMessage('75% complete ...')

                    number = number + 1                    
                
                rows2.UpdateRow(row2)        # row actualization
                row2 = rows2.Next()
            rows2 = None

            start1 = row1.INPUT_FID          # step to next value         
            sum = 0
                
        else:    # else jump straight to next value
            start1 = row1.INPUT_FID       
            sum = 0
                
    row1 = rows1.Next()
rows1 = None

# because the last "else" cycle did not pass: the same opration once more
if sum <> 0:

    if not dist == 0:
        fraction = area/dist**2.0
        sum = sum + fraction
            
    rows3 = gp.UpdateCursor(InPolygFC)
    row3 = rows3.Next()
    while row3:
        if row3.FID == start1:
            row3.SetValue(indexField, sum)
            rows3.UpdateRow(row3)
    
        row3 = rows3.Next()
    rows3 = None

#------------------------------------------------------------------------------------
# deleting temporary files
neighbours_dbf = "C:\\temp\\neighbours.dbf"
gp.Delete_management(neighbours_dbf)
gp.delete_management(centroids)

gp.AddMessage("\n Index values are in attribute table"'\n')

gp = None